--[[
Name: LibAuraInfo-1.0
Author(s): Cyprias (cyprias@gmail.com)
Documentation: http://www.wowace.com/addons/libaurainfo-1-0/
SVN: svn://svn.wowace.com/wow/libaurainfo-1-0/mainline/trunk
Description: Database of spellID's duration and debuff type.
Dependencies: LibStub
]]

local _G = _G
local print = print
local UnitAura = UnitAura
local _
local math_floor = math.floor
local UnitIsUnit = UnitIsUnit
local UnitExists = UnitExists
local UnitName = UnitName
local UnitGUID = UnitGUID

local MAJOR, MINOR = "LibAuraInfo-1.0", 1
if not LibStub then error(MAJOR .. " requires LibStub.") return end

local lib = LibStub:NewLibrary(MAJOR, MINOR)
if not lib then return end

lib.callbacks = lib.callbacks or LibStub("CallbackHandler-1.0"):New(lib)
if not lib.callbacks then	error(MAJOR .. " CallbackHandler-1.0.") return end




lib.GUIDDurations = {}

local DEBUG = false
--[===[@debug@
DEBUG = true
--@end-debug@]===]

local function debugPrint(...)
	if DEBUG then
		print(...)
	end
end
lib.debugPrint = debugPrint

--Save debuffType as a number, then return as a string when requested.
lib.debuffTypes = {
	Magic = 1,
	Disease = 2,
	Poison = 3,
	Curse = 4,
	[1] = "Magic",
	[2] = "Disease",
	[3] = "Poison",
	[4] = "Curse",
}

--~ lib.callbacks:Fire("LibAuraInfo-1.0_xxxx", "blaw")
function lib.callbacks:OnUsed(target, eventname)
	debugPrint("OnUsed", target, eventname)
end

function lib.callbacks:OnUnused(target, eventname)
	debugPrint("OnUsed", target, eventname)
end


local LAI_DB
local function Round(num, zeros)
	return math_floor( num * 10 ^ (zeros or 0) + 0.5 ) / 10 ^ (zeros or 0)
end
local function CheckUnitAuras(unitID, filterType)
--~ 	name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId  = UnitAura("unit", index or "name"[, "rank"[, "filter"]]) 
	local i = 1;
	local name, rank, dispelType, duration, isStealable, spellID, unitCaster
	
	local dstGUID = UnitGUID(unitID)
	local srcGUID
	lib:RemoveAllAurasFromGUID(dstGUID)
	while true do 
		name, rank, _, _, dispelType, duration, _, unitCaster, isStealable, _, spellID = UnitAura(unitID, i, filterType)
		if not name then break end

--~ 		if name == "Cyclone" then
--~ 			debugPrint(name, spellID, name, rank, duration)
--~ 		end

		duration = Round(duration)
		if not lib.auraInfo[spellID] then
			lib.auraInfo[spellID] = (duration or 0) .. ";" .. (dispelType and lib.debuffTypes[dispelType] or 0)--add it temporarily.
			
			--[===[@debug@
			if not LAI_DB[spellID] then
				--Save the spell info to a saved variable to add it later. End users won't execute this code.
				LAI_DB[spellID] = lib.auraInfo[spellID]    .."  --"..name .. (rank and rank ~= "" and " ("..rank..")" or "") .."*"
				debugPrint("Missing info on", spellID, dispelType, duration, name, rank)
			end
			--@end-debug@]===]
		else
			if unitCaster and UnitExists(unitCaster) then
				local GUID = UnitGUID(unitCaster)
				local baseDuration = lib:GetDuration(spellID)
				if baseDuration and baseDuration ~= duration then
					if not lib.GUIDDurations[GUID.."-"..spellID] then
						debugPrint("duration mismatch", UnitName(unitCaster), name, rank, "base:"..baseDuration, "his:"..duration)
						lib.GUIDDurations[GUID.."-"..spellID] = duration
					end
					
					--[===[@debug@
					if duration < baseDuration then
						--Save the spell info to a saved variable to add it later. End users won't execute this code.
						lib.auraInfo[spellID] = (duration or 0) .. ";" .. (dispelType and lib.debuffTypes[dispelType] or 0)--add it temporarily.
						LAI_DB[spellID] = lib.auraInfo[spellID]    .."  --"..name .. (rank and rank ~= "" and " ("..rank..")" or "") .."* REPLACE"
--~ 						if not LAI_DB[spellID] then
							debugPrint("replace info on", spellID, dispelType, duration, name, rank)
--~ 						end
					end
					--@end-debug@]===]
					
				end
			end
		end
		
		if unitCaster then
			srcGUID = UnitGUID(unitCaster)
			AddAuraToGUID(dstGUID, spellID, srcGUID, filterType == "HARMFUL")
		end
		
		i = i + 1
	end
end
--[[
local function OnEvent(self, event, ...)
	--I plan on removing this once we get near to release.
	if event == "UPDATE_MOUSEOVER_UNIT" then
		CheckUnitAuras("mouseover", "HELPFUL")
		CheckUnitAuras("mouseover", "HARMFUL")
	elseif event == "PLAYER_TARGET_CHANGED" then
		CheckUnitAuras("target", "HELPFUL")
		CheckUnitAuras("target", "HARMFUL")
	elseif event == "UNIT_TARGET" then
		local unitID = ...
		if not UnitIsUnit(unitID, "player") and UnitExists(unitID.."target") then
			CheckUnitAuras(unitID.."target", "HELPFUL")
			CheckUnitAuras(unitID.."target", "HARMFUL")
		end
	elseif event == "UNIT_AURA" then
		local unitID = ...
		if UnitExists(unitID) then
			CheckUnitAuras(unitID, "HELPFUL")
			CheckUnitAuras(unitID, "HARMFUL")
		end
	elseif event == "PLAYER_LOGOUT" then
		--[===[@debug@
		_G.LAI_DB = LAI_DB
		--@end-debug@]===]
	elseif event == "ADDON_LOADED" or event == "PLAYER_LOGIN" then
		--[===[@debug@
		LAI_DB = _G.LAI_DB or {}
		self:RegisterEvent('PLAYER_LOGOUT')
		--@end-debug@]===]
		self:RegisterEvent('UPDATE_MOUSEOVER_UNIT')
		self:RegisterEvent('PLAYER_TARGET_CHANGED')
		self:RegisterEvent('UNIT_TARGET')
		self:RegisterEvent('UNIT_AURA')
		
	end
end]]

lib.frame = lib.frame or CreateFrame("Frame")
local function OnEvent(self, event, ...)
	if self[event] then
		self[event](self, event, ...)
	end
end

function lib.frame:UPDATE_MOUSEOVER_UNIT()
	CheckUnitAuras("mouseover", "HELPFUL")
	CheckUnitAuras("mouseover", "HARMFUL")
end

function lib.frame:PLAYER_TARGET_CHANGED()
	CheckUnitAuras("target", "HELPFUL")
	CheckUnitAuras("target", "HARMFUL")
end

function lib.frame:UNIT_TARGET(unitID)
	CheckUnitAuras(unitID, "HELPFUL")
	CheckUnitAuras(unitID, "HARMFUL")
end

function lib.frame:UNIT_AURA(unitID)
	CheckUnitAuras(unitID, "HELPFUL")
	CheckUnitAuras(unitID, "HARMFUL")
end

function lib.frame:PLAYER_LOGOUT(unitID)
	--[===[@debug@
	_G.LAI_DB = LAI_DB
	--@end-debug@]===]
end

function lib.frame:ADDON_LOADED()
	--[===[@debug@
	LAI_DB = _G.LAI_DB or {}
	self:RegisterEvent('PLAYER_LOGOUT')
	--@end-debug@]===]
	self:RegisterEvent('UPDATE_MOUSEOVER_UNIT')
	self:RegisterEvent('PLAYER_TARGET_CHANGED')
	self:RegisterEvent('UNIT_TARGET')
	self:RegisterEvent('UNIT_AURA')
end

function lib.frame:PLAYER_LOGIN()
	self:ADDON_LOADED()
end

lib.frame:SetScript("OnEvent", OnEvent)
lib.frame:RegisterEvent('ADDON_LOADED')
lib.frame:RegisterEvent('PLAYER_LOGIN')
